/*
 * Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Peter Schiffer <pschiffe@redhat.com>
 */

#include <konkret/konkret.h>
#include "LMI_BIOSFeatureBIOSElements.h"
#include "utils.h"
#include "dmidecode.h"

static const CMPIBroker* _cb;

static void LMI_BIOSFeatureBIOSElementsInitialize(const CMPIContext *ctx)
{
    lmi_init(provider_name, _cb, ctx, provider_config_defaults);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    LMI_BIOSFeatureBIOSElements lmi_assoc;
    LMI_BIOSElementRef lmi_bios;
    LMI_BIOSFeatureRef lmi_feat;
    const char *ns = KNameSpace(cop);
    char id[BUFLEN];
    DmiBios dmi_bios;

    if (dmi_get_bios(&dmi_bios) != 0) {
        goto done;
    }

    snprintf(id, BUFLEN, "%s BIOS %s", dmi_bios.vendor, dmi_bios.version);

    LMI_BIOSElementRef_Init(&lmi_bios, _cb, ns);
    LMI_BIOSElementRef_Set_SoftwareElementState(&lmi_bios,
            LMI_BIOSElement_SoftwareElementState_Running);
    LMI_BIOSElementRef_Set_TargetOperatingSystem(&lmi_bios,
            LMI_BIOSElement_TargetOperatingSystem_Not_Applicable);
    LMI_BIOSElementRef_Set_Version(&lmi_bios, dmi_bios.version);
    LMI_BIOSElementRef_Set_Name(&lmi_bios, id);
    LMI_BIOSElementRef_Set_SoftwareElementID(&lmi_bios, id);

    LMI_BIOSFeatureRef_Init(&lmi_feat, _cb, ns);
    LMI_BIOSFeatureRef_Set_Name(&lmi_feat, id);
    LMI_BIOSFeatureRef_Set_ProductName(&lmi_feat, id);
    LMI_BIOSFeatureRef_Set_Vendor(&lmi_feat, dmi_bios.vendor);
    LMI_BIOSFeatureRef_Set_Version(&lmi_feat, dmi_bios.version);
    LMI_BIOSFeatureRef_Set_IdentifyingNumber(&lmi_feat, dmi_bios.version);

    LMI_BIOSFeatureBIOSElements_Init(&lmi_assoc, _cb, ns);
    LMI_BIOSFeatureBIOSElements_Set_GroupComponent(&lmi_assoc, &lmi_feat);
    LMI_BIOSFeatureBIOSElements_Set_PartComponent(&lmi_assoc, &lmi_bios);

    KReturnInstance(cr, lmi_assoc);

done:
    dmi_free_bios(&dmi_bios);

    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char**properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsAssociationCleanup(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_BIOSFeatureBIOSElements_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_BIOSFeatureBIOSElements_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_BIOSFeatureBIOSElements_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus LMI_BIOSFeatureBIOSElementsReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_BIOSFeatureBIOSElements_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub(
    LMI_BIOSFeatureBIOSElements,
    LMI_BIOSFeatureBIOSElements,
    _cb,
    LMI_BIOSFeatureBIOSElementsInitialize(ctx))

CMAssociationMIStub(
    LMI_BIOSFeatureBIOSElements,
    LMI_BIOSFeatureBIOSElements,
    _cb,
    LMI_BIOSFeatureBIOSElementsInitialize(ctx))

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_BIOSFeatureBIOSElements",
    "LMI_BIOSFeatureBIOSElements",
    "instance association")
