/*
 * Copyright (C) 2014 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Jan Synacek <jsynacek@redhat.com>
 *
 */

#include <konkret/konkret.h>
#include "LMI_HostedSELinuxService.h"
#include "selinux.h"

static const CMPIBroker *_cb;

static void LMI_HostedSELinuxServiceInitialize(const CMPIContext *ctx)
{
    lmi_init(provider_name, _cb, ctx, provider_config_defaults);
}

static CMPIStatus LMI_HostedSELinuxServiceCleanup(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_HostedSELinuxServiceEnumInstanceNames(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_HostedSELinuxServiceEnumInstances(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char **properties)
{
    const char *ns = KNameSpace(cop);
    CMPIStatus st;
    LMI_HostedSELinuxService lmi_hosted;
    LMI_SELinuxService lmi_service;
    CMPIObjectPath *o;

    LMI_HostedSELinuxService_Init(&lmi_hosted, _cb, ns);
    LMI_HostedSELinuxService_SetObjectPath_Antecedent(&lmi_hosted, lmi_get_computer_system_safe(cc));

    init_selinux_service(&lmi_service, _cb, cc, ns);

    o = LMI_SELinuxService_ToObjectPath(&lmi_service, &st);
    lmi_return_if_status_not_ok(st);
    LMI_HostedSELinuxService_SetObjectPath_Dependent(&lmi_hosted, o);

    KReturnInstance(cr, lmi_hosted);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_HostedSELinuxServiceGetInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char **properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_HostedSELinuxServiceCreateInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const CMPIInstance *ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_HostedSELinuxServiceModifyInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const CMPIInstance *ci,
    const char**properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_HostedSELinuxServiceDeleteInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_HostedSELinuxServiceExecQuery(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *lang,
    const char *query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_HostedSELinuxServiceAssociationCleanup(
    CMPIAssociationMI *mi,
    const CMPIContext *cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_HostedSELinuxServiceAssociators(
    CMPIAssociationMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *assocClass,
    const char *resultClass,
    const char *role,
    const char *resultRole,
    const char **properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_HostedSELinuxService_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus LMI_HostedSELinuxServiceAssociatorNames(
    CMPIAssociationMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *assocClass,
    const char *resultClass,
    const char *role,
    const char *resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_HostedSELinuxService_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus LMI_HostedSELinuxServiceReferences(
    CMPIAssociationMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *assocClass,
    const char *role,
    const char **properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_HostedSELinuxService_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus LMI_HostedSELinuxServiceReferenceNames(
    CMPIAssociationMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *assocClass,
    const char *role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_HostedSELinuxService_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub(
    LMI_HostedSELinuxService,
    LMI_HostedSELinuxService,
    _cb,
    LMI_HostedSELinuxServiceInitialize(ctx))

CMAssociationMIStub(
    LMI_HostedSELinuxService,
    LMI_HostedSELinuxService,
    _cb,
    LMI_HostedSELinuxServiceInitialize(ctx))

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_HostedSELinuxService",
    "LMI_HostedSELinuxService",
    "instance association")
