/*
 * Copyright (C) 2014 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Jan Synacek <jsynacek@redhat.com>
 *
 */

#include <konkret/konkret.h>
#include "LMI_SELinuxMethodResult.h"
#include "selinux.h"
#include "job_manager.h"

static const CMPIBroker *_cb = NULL;

static void LMI_SELinuxMethodResultInitialize(const CMPIContext *ctx)
{
    selinux_provider_init(LMI_SELinuxMethodResult_ClassName, FALSE, provider_config_defaults, _cb, ctx);
}

static CMPIStatus LMI_SELinuxMethodResultCleanup(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    CMPIBoolean term)
{
    return jobmgr_cleanup(LMI_SELinuxMethodResult_ClassName);
}

static CMPIStatus LMI_SELinuxMethodResultEnumInstanceNames(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_SELinuxMethodResultEnumInstances(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char **properties)
{
    CMPIStatus status = {CMPI_RC_OK, NULL};
    CMPIInstance *inst;
    LmiJob *job;
    guint *jobnum;
    guint *jobnums = jobmgr_get_job_numbers(NULL);

    if (jobnums == NULL)
        CMReturn(CMPI_RC_ERR_FAILED);

    for (jobnum = jobnums; *jobnum != 0; ++jobnum) {
        job = jobmgr_get_job_by_number(*jobnum);
        if (!job)
            /* job may have been deleted since numbers query */
            continue;

        status = jobmgr_job_to_method_result_instance(job, NULL, &inst);
        if (status.rc) {
            gchar *jobid = lmi_job_get_jobid(job);
            if (status.msg) {
                lmi_warn("Failed to make method result instance out of job"
                       " \"%s\": %s", jobid, CMGetCharsPtr(status.msg, NULL));
            } else {
                lmi_warn("Failed to make method result instance out of job"
                       " \"%s\"", jobid);
            }
            g_free(jobid);
        } else {
            CMReturnInstance(cr, inst);
        }
        g_clear_object(&job);
    }

    g_free(jobnums);

    return status;
}

static CMPIStatus LMI_SELinuxMethodResultGetInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char **properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_SELinuxMethodResultCreateInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const CMPIInstance *ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_SELinuxMethodResultModifyInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const CMPIInstance *ci,
    const char **properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_SELinuxMethodResultDeleteInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_SELinuxMethodResultExecQuery(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *lang,
    const char *query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    LMI_SELinuxMethodResult,
    LMI_SELinuxMethodResult,
    _cb,
    LMI_SELinuxMethodResultInitialize(ctx))

static CMPIStatus LMI_SELinuxMethodResultMethodCleanup(
    CMPIMethodMI *mi,
    const CMPIContext *cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_SELinuxMethodResultInvokeMethod(
    CMPIMethodMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *meth,
    const CMPIArgs *in,
    CMPIArgs *out)
{
    return LMI_SELinuxMethodResult_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    LMI_SELinuxMethodResult,
    LMI_SELinuxMethodResult,
    _cb,
    LMI_SELinuxMethodResultInitialize(ctx))

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_SELinuxMethodResult",
    "LMI_SELinuxMethodResult",
    "instance method")
