/*
 * Copyright (C) 2014 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Jan Synacek <jsynacek@redhat.com>
 *
 */

#include <konkret/konkret.h>
#include "LMI_SELinuxServiceHasElement.h"
#include "LMI_SELinuxBoolean.h"
#include "LMI_SELinuxPort.h"
#include "selinux.h"

static const CMPIBroker *_cb;

static void LMI_SELinuxServiceHasElementInitialize(const CMPIContext *ctx)
{
    lmi_init(provider_name, _cb, ctx, provider_config_defaults);
}

static CMPIStatus LMI_SELinuxServiceHasElementCleanup(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_SELinuxServiceHasElementEnumInstanceNames(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_SELinuxServiceHasElementEnumInstances(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char **properties)
{
    const char *ns = KNameSpace(cop);
    CMPIStatus st;

    LMI_SELinuxServiceHasElement lmi_sshe;
    LMI_SELinuxServiceHasElement_Init(&lmi_sshe, _cb, ns);

    LMI_SELinuxService lmi_service;
    init_selinux_service(&lmi_service, _cb, cc, ns);

    CMPIObjectPath *o = LMI_SELinuxService_ToObjectPath(&lmi_service, &st);
    lmi_return_if_status_not_ok(st);
    LMI_SELinuxServiceHasElement_SetObjectPath_Dependent(&lmi_sshe, o);

    /* booleans */
    GSList *booleans = lmi_get_selinux_boolean_names();
    GSList *iter;
    for (iter = booleans; iter; iter = iter->next) {
        LMI_SELinuxBooleanRef boolRef;

        LMI_SELinuxBooleanRef_Init(&boolRef, _cb, ns);
        LMI_SELinuxBooleanRef_Set_InstanceID(&boolRef, (const char *) iter->data);
        LMI_SELinuxServiceHasElement_SetObjectPath_Antecedent(&lmi_sshe,
                LMI_SELinuxBooleanRef_ToObjectPath(&boolRef, &st));
        KReturnInstance(cr, lmi_sshe);
    }
    g_slist_free_full(booleans, free);

    /* ports */
    GSList *ports = lmi_get_selinux_port_names();
    for (iter = ports; iter; iter = iter->next) {
        LMI_SELinuxPortRef portRef;

        LMI_SELinuxPortRef_Init(&portRef, _cb, ns);
        LMI_SELinuxPortRef_Set_InstanceID(&portRef, (const char *) iter->data);
        LMI_SELinuxServiceHasElement_SetObjectPath_Antecedent(&lmi_sshe,
                LMI_SELinuxPortRef_ToObjectPath(&portRef, &st));
        KReturnInstance(cr, lmi_sshe);
    }
    g_slist_free_full(ports, free);

    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_SELinuxServiceHasElementGetInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char **properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_SELinuxServiceHasElementCreateInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const CMPIInstance *ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_SELinuxServiceHasElementModifyInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const CMPIInstance *ci,
    const char**properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_SELinuxServiceHasElementDeleteInstance(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_SELinuxServiceHasElementExecQuery(
    CMPIInstanceMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *lang,
    const char *query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_SELinuxServiceHasElementAssociationCleanup(
    CMPIAssociationMI *mi,
    const CMPIContext *cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_SELinuxServiceHasElementAssociators(
    CMPIAssociationMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *assocClass,
    const char *resultClass,
    const char *role,
    const char *resultRole,
    const char **properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_SELinuxServiceHasElement_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus LMI_SELinuxServiceHasElementAssociatorNames(
    CMPIAssociationMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *assocClass,
    const char *resultClass,
    const char *role,
    const char *resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_SELinuxServiceHasElement_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus LMI_SELinuxServiceHasElementReferences(
    CMPIAssociationMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *assocClass,
    const char *role,
    const char **properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_SELinuxServiceHasElement_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus LMI_SELinuxServiceHasElementReferenceNames(
    CMPIAssociationMI *mi,
    const CMPIContext *cc,
    const CMPIResult *cr,
    const CMPIObjectPath *cop,
    const char *assocClass,
    const char *role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_SELinuxServiceHasElement_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub(
    LMI_SELinuxServiceHasElement,
    LMI_SELinuxServiceHasElement,
    _cb,
    LMI_SELinuxServiceHasElementInitialize(ctx))

CMAssociationMIStub(
    LMI_SELinuxServiceHasElement,
    LMI_SELinuxServiceHasElement,
    _cb,
    LMI_SELinuxServiceHasElementInitialize(ctx))

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_SELinuxServiceHasElement",
    "LMI_SELinuxServiceHasElement",
    "instance association")
